/** 
 * 
 * Copyright 2004 Protique Ltd
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.activemq.ws.notification.impl;

import EDU.oswego.cs.dl.util.concurrent.Slot;
import org.activemq.ws.notification.NotificationConsumer;
import org.activemq.ws.xmlbeans.notification.base.NotifyDocument;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * A simple NotificationConsumer used for unit testing.
 *
 * @version $Revision: 1.1 $
 */
public class StubNotificationConsumer implements NotificationConsumer {
    private static final transient Log log = LogFactory.getLog(StubNotificationConsumer.class);

    private Slot result;
    private int timeout;

    public StubNotificationConsumer() {
        this(new Slot());
    }

    public StubNotificationConsumer(Slot result) {
        this.result = result;
    }

    public void notify(NotifyDocument notify) {
        try {
            timeout = 1000;
            result.offer(notify, timeout);
        }
        catch (InterruptedException e) {
            log.warn("Could not offer notification: " + e, e);
        }
    }

    public Slot getResult() {
        return result;
    }

    public int getTimeout() {
        return timeout;
    }

    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }
}
